#pragma once

#define PROJECTM_VERSION_MAJOR 4
#define PROJECTM_VERSION_MINOR 1
#define PROJECTM_VERSION_PATCH 6

/* Full version number as a string literal */
#define PROJECTM_VERSION_STRING "4.1.6"

/* VCS version number, e.g. Git commit hash */
#define PROJECTM_VERSION_VCS "Unknown"
